# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2020 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS -> Catapult Inventory Workbook
"""

import logging

from sqlalchemy import orm

from corepos.db.office_op import model as corepos

from rattail.gpc import GPC
from rattail.util import OrderedDict
from rattail.importing.handlers import ToFileHandler
from rattail_corepos.corepos.importing.db.corepos import FromCoreHandler, FromCore
from rattail_onager.catapult.importing import inventory as catapult_importing


log = logging.getLogger(__name__)


class FromCoreToCatapult(FromCoreHandler, ToFileHandler):
    """
    Handler for CORE -> Catapult (Inventory Workbook)
    """
    host_title = "CORE-POS"
    local_title = "Catapult (Inventory Workbook)"
    direction = 'export'

    def get_importers(self):
        importers = OrderedDict()
        importers['InventoryItem'] = InventoryItemImporter
        return importers


class InventoryItemImporter(FromCore, catapult_importing.model.InventoryItemImporter):
    """
    Inventory Item data importer.
    """
    host_model_class = corepos.Product
    supported_fields = [
        'item_id',
        'dept_id',
        'dept_name',
        'receipt_alias',
        'brand',
        'item_name',
        'size',
        # 'sugg_retail',
        'last_cost',
        'price_divider',
        'base_price',
        # 'disc_mult',
        'ideal_margin',
        'bottle_deposit',
        'pos_menu_group',
        # 'scale_label',
        'sold_by_ea_or_lb',
        'quantity_required',
        'weight_profile',
        'tax_1',
        'tax_2',
        'spec_tend_1',
        'spec_tend_2',
        'age_required',
        # 'location',
        # 'family_line',
        # 'alt_id',
        # 'alt_receipt_alias',
        # 'alt_pkg_qty',
        # 'alt_pkg_price',
        # 'auto_discount',
        # 'supplier_unit_id',
        # 'supplier_id',
        # 'unit',
        # 'num_pkgs',
        # 'cs_pk_multiplier',
        # 'dsd',
        # 'pf1',
        # 'pf2',
        # 'pf3',
        # 'pf4',
        # 'pf5',
        # 'pf6',
        # 'pf7',
        # 'pf8',
        # 'memo',
        # 'scale_shelf_life',
        # 'scale_shelf_life_type',
        # 'scale_ingredient_text',
    ]

    def setup(self):
        super(InventoryItemImporter, self).setup()

        self.ignored_upcs = self.config.getlist(
            'corepos', 'exporting.catapult_inventory.ignored_upcs')

        self.warn_missing_department = self.config.getbool(
            'corepos', 'exporting.catapult_inventory.warn_missing_department',
            default=True)

    def query(self):
        query = self.host_session.query(corepos.Product)\
                                 .order_by(corepos.Product.upc)\
                                 .options(orm.joinedload(corepos.Product.department))\
                                 .options(orm.joinedload(corepos.Product.subdepartment))
        return query

    def normalize_host_object(self, product):
        item_id = product.upc

        if not item_id:
            log.warning("product id %s has no upc: %s",
                        product.id, product)
            return

        if not item_id.isdigit():
            log.debug("product has non-numeric upc %s: %s",
                      item_id, product)
            return

        if self.ignored_upcs and item_id in self.ignored_upcs:
            log.debug("ignoring UPC %s for product: %s", item_id, product)
            return

        is_plu = False
        if len(str(int(item_id))) < 6:
            is_plu = True
            item_id = str(int(item_id))
        else: # must add check digit, and re-format
            upc = GPC(item_id, calc_check_digit='upc')
            item_id = str(upc)
            assert len(item_id) == 14
            # drop leading zero(s)
            if item_id[1] == '0': # UPC-A
                item_id = item_id[2:]
            else: # EAN13
                item_id = item_id[1:]

        department = product.department
        if not department:
            logger = log.warning if self.warn_missing_department else log.debug
            logger("item_id %s has no department: %s", item_id, product)
            return

        sold_by_ea_or_lb = None
        if is_plu:
            sold_by_ea_or_lb = 'LB' if product.scale else 'EA'

        weight_profile = None
        if product.scale and len(item_id) == 12 and item_id[0] == '2':
            weight_profile = 'LBNT'

        # TODO: need to finish the logic to map/calculate tax rates
        tax_1 = 0
        tax_2 = 0

        # # TODO: should let config drive these somehow, obviously
        # tax_rate_ids_1 = (1,)
        # tax_rate_ids_2 = (2,)
        # tax_component_ids_1 = (1,)
        # tax_component_ids_2 = (2,)

        # if product.tax_rate:

        #     if product.tax_rate.components:
        #         for component in product.tax_rate.components:
        #             if tax_component_ids_1 and component.id in tax_component_ids_1:
        #                 tax_1 += component.rate
        #             if tax_component_ids_2 and component.id in tax_component_ids_2:
        #                 tax_2 += component.rate

        #     else: # no components
        #         rate = product.tax_rate
        #         if tax_rate_ids_1 and rate.id in tax_rate_ids_1:
        #             tax_1 += rate.rate
        #         if tax_rate_ids_2 and rate.id in tax_rate_ids_2:
        #             tax_2 += rate.rate

        return {
            'item_id': item_id,
            'dept_id': department.number,
            'dept_name': department.name,
            'receipt_alias': product.description,
            'brand': product.brand,
            'item_name': product.description,
            'size': product.size,

            # TODO: does CORE have this?
            # 'sugg_retail': None,

            'last_cost': product.cost,
            'price_divider': product.quantity or 1, # TODO: is this right?
            'base_price': product.normal_price,
            'ideal_margin': department.margin * 100 if department.margin else None,

            # TODO: does CORE have these?
            # 'disc_mult': None,

            'bottle_deposit': product.deposit or None,
            'pos_menu_group': product.subdepartment.name if product.subdepartment else None,

            # TODO: does CORE have these?
            # 'scale_label': None,

            'sold_by_ea_or_lb': sold_by_ea_or_lb,
            'quantity_required': 'Y' if product.quantity_enforced else None,
            'weight_profile': weight_profile,
            'tax_1': tax_1 or None, # TODO: logic above is unfinished
            'tax_2': tax_2 or None, # TODO: logic above is unfinished
            'spec_tend_1': 'EBT' if product.foodstamp else None,
            'spec_tend_2': 'WIC' if product.wicable else None,
            'age_required': product.id_enforced or None,

            # TODO: does CORE have these?
            # 'location': None,
            # 'family_line': None,

            # TODO: does CORE have these?
            # 'alt_id': None,
            # 'alt_receipt_alias': None,
            # 'alt_pkg_qty': None,
            # 'alt_pkg_price': None,
            # 'auto_discount': None,

            # TODO: pretty sure CORE has these, but i'm not sure where
            # 'supplier_unit_id': None,
            # 'supplier_id': None,
            # 'unit': None,
            # 'num_pkgs': None,
            # 'cs_pk_multiplier': None,

            # TODO: does CORE have this?
            # 'dsd': None,

            # TODO: are these needed?
            # 'pf1',
            # 'pf2',
            # 'pf3',
            # 'pf4',
            # 'pf5',
            # 'pf6',
            # 'pf7',
            # 'pf8',

            # TODO: does CORE have these?
            # 'memo': None,
            # 'scale_shelf_life',
            # 'scale_shelf_life_type',
            # 'scale_ingredient_text',
        }
